/*
   FNT - SQL-backend for MySQL - function connect

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <mysql.h>
#include <stdlib.h>
#include <string.h>

#include "../include/global.h"

#include "../include/types.h"
#include "../include/error.h"
#include "../include/plugin.h"
#include "../include/sql.h"

#include "main.h"

/*** global variables **************************************************/

/*** file scope macro definitions **************************************/

/*** file scope type declarations **************************************/

/*** file scope variables **********************************************/

/*** file scope functions **********************************************/

/*** public functions **************************************************/

DLLEXPORT fnt_sql_connect_t *
fntsql_connect (void)
{
    fnt_sql_connect_t *sql_connect;
    MYSQL *conn = NULL;
    MYSQL *real_conn = NULL;

    int connect_type;
    bool autoreconnect = true;
    char *unix_socket;
    char *db_name;
    char *host_name;
    char *port_number;
    unsigned int port;

    conn = mysql_init (NULL);
    if (conn == NULL)
    {
	fnt_error_push (FNT_MODULE_sql, FNT_SQL_E_DRIVER,
			_("Mysql - fail to init connection"));
	return NULL;
    }

    if (connect_info.is_local_path)
    {
	connect_type = MYSQL_PROTOCOL_SOCKET;
	unix_socket = connect_info.local_db_path;
	host_name = NULL;
	port = 0;
    }
    else
    {
	connect_type = MYSQL_PROTOCOL_TCP;
	unix_socket = NULL;
	host_name = connect_info.host_name;
	port = connect_info.port_number;
    }
    mysql_options (conn, MYSQL_OPT_PROTOCOL, (void *) &connect_type);
    mysql_options (conn, MYSQL_OPT_RECONNECT, (my_bool *) & autoreconnect);
    mysql_options (conn, MYSQL_SET_CHARSET_NAME, "utf8");
    mysql_options (conn, MYSQL_INIT_COMMAND, "set names 'utf8';");

    real_conn = mysql_real_connect (conn,
				    host_name,
				    connect_info.user_name,
				    connect_info.password,
				    NULL, port, unix_socket, 0);

    if (real_conn == NULL)
    {
	fnt_error_push (FNT_MODULE_sql, FNT_SQL_E_DRIVER,
			_("Mysql - fail to connect: '%s'"),
			mysql_error (conn));
	mysql_close (conn);
	return NULL;
    }
    if (mysql_select_db (real_conn, connect_info.db_name) != 0)
    {
	mysql_close (conn);
	return NULL;
    }

    sql_connect = g_try_malloc (sizeof (fnt_sql_connect_t));
    if (sql_connect == NULL)
    {
	mysql_close (conn);
	return NULL;
    }
    sql_connect->db_handle = (void *) real_conn;

    sql_connect->info = g_try_malloc (sizeof (fnt_sql_connect_info_t));
    if (sql_connect->info == NULL)
    {
	mysql_close (conn);
	g_free (sql_connect);
	return NULL;
    }
    memcpy ((void *) sql_connect->info, (void *) &connect_info,
	    sizeof (fnt_sql_connect_info_t));

    sql_connect->info->local_db_path =
	g_strdup ((unix_socket ? unix_socket : ""));

    sql_connect->info->db_name =
	g_strdup ((connect_info.db_name) ? connect_info.db_name : "");

    sql_connect->info->host_name = g_strdup ((host_name) ? host_name : "");

    sql_connect->info->user_name =
	g_strdup ((connect_info.user_name) ? connect_info.user_name : "");

    sql_connect->info->password =
	g_strdup ((connect_info.password) ? connect_info.password : "");

    sql_connect->info->port_number = port;

    mysql_autocommit (real_conn, 0);

    return sql_connect;
}
